﻿using System;
using MoonTools.ECS.Collections;

namespace MoonTools.ECS
{
	public struct FilterBuilder
	{
		World World;
		IndexableSet<TypeId> Included;
		IndexableSet<TypeId> Excluded;

		internal FilterBuilder(World world)
		{
			World = world;
			Included = new IndexableSet<TypeId>();
			Excluded = new IndexableSet<TypeId>();
		}

		private FilterBuilder(World world, IndexableSet<TypeId> included, IndexableSet<TypeId> excluded)
		{
			World = world;
			Included = included;
			Excluded = excluded;
		}

		public FilterBuilder Include<T>() where T : unmanaged
		{
			Included.Add(World.GetComponentTypeId<T>());
			return new FilterBuilder(World, Included, Excluded);
		}

		public FilterBuilder Exclude<T>() where T : unmanaged
		{
			Excluded.Add(World.GetComponentTypeId<T>());
			return new FilterBuilder(World, Included, Excluded);
		}

		public Filter Build()
		{
			var signature = new FilterSignature(Included, Excluded);
			return World.GetFilter(signature);
		}

		public Filter<T> Of<T>() where T : unmanaged
		{
			return new Filter<T>(Include<T>().Build());
		}

		public Filter<T1, T2> Of<T1, T2>() where T1 : unmanaged where T2 : unmanaged
		{
			return new Filter<T1, T2>(Include<T1>().Include<T2>().Build());
		}

		public Filter<T1, T2, T3> Of<T1, T2, T3>() where T1 : unmanaged where T2 : unmanaged where T3 : unmanaged
		{
			return new Filter<T1, T2, T3>(Include<T1>().Include<T2>().Include<T3>().Build());
		}
	}
}
